@@ -17,9 +17,10 @@ server = function(input, output, session) {
   options(shiny.maxRequestSize = 30 * 1024 ^ 2)
 
   myPars = reactiveValues(
+    print = FALSE,          # if TRUE, some functions print a message to the console when called
+    debugQn = FALSE,        # for debugging - click "?" to step into the code
     zoomFactor = 2,         # zoom buttons change time zoom by this factor
     zoomFactor_freq = 1.5,  # same for frequency
-    print = FALSE,          # if TRUE, some functions print a message to the console when called
     shinyTip_show = 1000,      # delay until showing a tip (ms)
     shinyTip_hide = 0,         # delay until hiding a tip (ms)
     initDur = 2000,            # initial duration to plot (ms)
@@ -33,9 +34,9 @@ server = function(input, output, session) {
     listen_enter = FALSE,      # enable/disable ENTER to close modal (new annotation)
     listen_enter_edit = FALSE, # ENTER to edit an existing annotation
     cursor = 0,
-    play = list(on = FALSE),
-    debugQn = FALSE             # for debugging - click "?" to step into the code
+    play = list(on = FALSE)
   )
+  tooltip_options = list(delay = list(show = 1000, hide = 0))
 
   # NB: using myPars$play$cursor for some reason invalidates the observer,
   # so it keeps executing as fast as it can - no idea why!
@@ -59,6 +60,7 @@ server = function(input, output, session) {
   })
   observeEvent(input$append, {
     myPars$out = try(read.csv('www/temp.csv', stringsAsFactors = FALSE))
+    myPars$out = unique(myPars$out)  # remove duplicate rows
     removeModal()
   })
 
@@ -77,6 +79,7 @@ server = function(input, output, session) {
     myPars$selection = NULL
     myPars$cursor = 0
     myPars$spectrogram_brush = NULL
+    shinyjs::js$clearBrush(s = '_brush')
   }
 
   resetSliders = function() {
@@ -212,6 +215,14 @@ server = function(input, output, session) {
     #                   value = c(0, min(def_form['spectrum_xlim', 'default'], myPars$nyquist)),
     #                   max = myPars$nyquist)
     myPars$dur = length(myPars$temp_audio@left) * 1000 / myPars$temp_audio@samp.rate
+    myPars$myAudio_list = list(
+      sound = myPars$myAudio,
+      samplingRate = myPars$samplingRate,
+      scale = myPars$maxAmpl,
+      timeShift = 0,
+      ls = length(myPars$myAudio),
+      duration = myPars$dur / 1000
+    )
     myPars$time = seq(1, myPars$dur, length.out = myPars$ls)
     myPars$spec_xlim = c(0, min(myPars$initDur, myPars$dur))
     if (!is.finite(myPars$spec_xlim[2])) browser()  # weird glitches
@@ -251,9 +262,8 @@ server = function(input, output, session) {
     # matrix and re-draw manually with soundgen:::filled.contour.mod
     if (!is.null(myPars$myAudio)) {  # & is.null(myPars$spec)
       if (myPars$print) print('Extracting spectrogram...')
-      temp_spec = try(spectrogram(
-        myPars$myAudio,
-        samplingRate = myPars$samplingRate,
+      temp_spec = try(soundgen:::.spectrogram(
+        myPars$myAudio_list,
         dynamicRange = input$dynamicRange,
         windowLength = input$windowLength,
         step = input$step,
@@ -261,13 +271,17 @@ server = function(input, output, session) {
         zp = 2 ^ input$zp,
         contrast = input$specContrast,
         brightness = input$specBrightness,
-        output = 'processed',
+        blur = c(input$blur_freq, input$blur_time),
+        specType = input$specType,
+        output = 'all',
         plot = FALSE
       ))
-      if (class(temp_spec) != 'try-error' &&
+      if (!inherits(temp_spec, 'try-error') &&
           length(temp_spec) > 0 &&
-          is.matrix(temp_spec))
-        myPars$spec = temp_spec
+          is.matrix(temp_spec$processed)) {
+        myPars$spec = temp_spec$processed
+        myPars$reassigned = temp_spec$reassigned
+      }
     }
   })
 
@@ -287,7 +301,8 @@ server = function(input, output, session) {
     # this is the new sound file. NB: has to be saved in www/ !!!
     seewave::savewav(myPars$temp_audio,
                      f = myPars$samplingRate,
-                     filename = paste0('www/', myPars$myfile))
+                     filename = paste0('www/', myPars$myfile),
+                     extensible = FALSE)
     output$htmlAudio = renderUI(
       tags$audio(src = myPars$myfile, type = myPars$myAudio_type,
                  id = 'myAudio',
@@ -388,28 +403,22 @@ server = function(input, output, session) {
 
   ## SPECTROGRAM
   output$spectrogram = renderPlot({
-    if (!is.null(myPars$spec)) {
-      if (myPars$print) print('Drawing spectrogram...')
-      par(mar = c(0.2, 2, 0.5, 2))  # no need to save user's graphical par-s - revert to orig on exit
-      if (is.null(myPars$spec)) {
-        plot(1:10, type = 'n', bty = 'n', axes = FALSE, xlab = '', ylab = '')
-        text(x = 5, y = 5,
-             labels = 'Upload wav/mp3 file(s) to begin...\nSuggested max duration ~30 s')
-      } else {
-        if (input$spec_colorTheme == 'bw') {
-          color.palette = function(x) gray(seq(from = 1, to = 0, length = x))
-        } else if (input$spec_colorTheme == 'seewave') {
-          color.palette = seewave::spectro.colors
-        } else {
-          colFun = match.fun(input$spec_colorTheme)
-          color.palette = function(x) rev(colFun(x))
-        }
+    if (myPars$print) print('Drawing spectrogram...')
+    par(mar = c(0.2, 2, 0.5, 2))  # no need to save user's graphical par-s - revert to orig on exit
+    if (is.null(myPars$spec)) {
+      plot(1:10, type = 'n', bty = 'n', axes = FALSE, xlab = '', ylab = '')
+      text(x = 5, y = 5, cex = 3,
+           labels =
+             'Upload wav/mp3 file(s) to begin...\n
+             Suggested max duration ~10 min')
+    } else {
+      if (input$specType != 'reassigned') {
+        # rasterized spectrogram
         soundgen:::filled.contour.mod(
           x = as.numeric(colnames(myPars$spec_trimmed)),
           y = as.numeric(rownames(myPars$spec_trimmed)),
           z = t(myPars$spec_trimmed),
-          levels = seq(0, 1, length = 30),
-          color.palette = color.palette,
+          col = soundgen:::switchColorTheme(input$spec_colorTheme)(input$nColors),
           log = if (input$spec_yScale == 'log') 'y' else '',
           yScale = if (input$spec_yScale %in% c('bark', 'mel', 'ERB')) input$spec_yScale else 'orig',
           xlim = myPars$spec_xlim,
@@ -419,34 +428,42 @@ server = function(input, output, session) {
           main = '',
           ylim = input$spec_ylim
         )
-
-        # Add text label of file name
-        if (input$spec_yScale == 'bark') {
-          spec_ylim = tuneR::hz2bark(input$spec_ylim * 1000)
-          nyquist = tuneR::hz2bark(myPars$samplingRate / 2)
-        } else if (input$spec_yScale == 'mel') {
-          spec_ylim = tuneR::hz2mel(input$spec_ylim * 1000)
-          nyquist = tuneR::hz2mel(myPars$samplingRate / 2)
-        } else if (input$spec_yScale == 'ERB') {
-          spec_ylim = HzToERB(input$spec_ylim * 1000)
-          nyquist = HzToERB(myPars$samplingRate / 2)
-        } else {
-          spec_ylim = input$spec_ylim
-          nyquist = myPars$samplingRate / 2000
-        }
-        if (spec_ylim[2] > nyquist) spec_ylim[2] = nyquist
-        text_y_lab = spec_ylim[2] - diff(spec_ylim) * .01
-        text(x = myPars$spec_xlim[1] + diff(myPars$spec_xlim) * .01,
-             y = text_y_lab,
-             labels = myPars$myAudio_filename,
-             adj = c(0, 1))  # left, top
+      } else {
+        # unrasterized reassigned spectrogram
+        soundgen:::plotUnrasterized(
+          myPars$reassigned,
+          col = soundgen:::switchColorTheme(input$spec_colorTheme)(input$nColors),
+          log = if (input$spec_yScale == 'log') 'y' else '',
+          yScale = if (input$spec_yScale %in% c('bark', 'mel', 'ERB'))
+            input$spec_yScale else 'orig',
+          xlim = myPars$spec_xlim,
+          xaxt = 'n',
+          xaxs = 'i', xlab = '',
+          ylab = '',
+          main = '',
+          ylim = input$spec_ylim,
+          cex = input$reass_cex
+        )
       }
+
+      # Add text label of file name
+      if (input$spec_yScale %in% c('bark', 'mel', 'ERB')) {
+        spec_ylim = HzToOther(input$spec_ylim * 1000, input$spec_yScale)
+        nyquist = HzToOther(myPars$samplingRate / 2, input$spec_yScale)
+      } else {
+        spec_ylim = input$spec_ylim
+        nyquist = myPars$samplingRate / 2000
+      }
+      if (spec_ylim[2] > nyquist) spec_ylim[2] = nyquist
+      text_y_lab = spec_ylim[2] - diff(spec_ylim) * .01
+      text(x = myPars$spec_xlim[1] + diff(myPars$spec_xlim) * .01,
+           y = text_y_lab,
+           labels = myPars$myAudio_filename,
+           adj = c(0, 1))  # left, top
     }
   })
 
   observeEvent(input$spectrogram_click, {
-    # myPars$spectrogram_brush = NULL
-    # shinyjs::js$clearBrush(s = '_brush')
     myPars$cursor = input$spectrogram_click$x
   })
 
@@ -507,7 +524,7 @@ server = function(input, output, session) {
                  axes = FALSE,
                  xlab = '', ylab = ''
             )
-            for (i in 1:nrow(myPars$ann)) {
+            for (i in seq_len(nrow(myPars$ann))) {
               r = rnorm(1, 0, .05)  # random vertical shift to avoid overlap
               # highlight current annotation
               highlight = ifelse(is.numeric(myPars$currentAnn) &&
@@ -653,7 +670,8 @@ server = function(input, output, session) {
 
     # save a backup in case the app crashes before done() fires
     temp = soundgen:::rbind_fill(myPars$out, myPars$ann)
-    temp = temp[order(temp$file), ]
+    temp = unique(temp[order(temp$file), ])  # remove duplicate rows
+    my_annot <<- temp
     write.csv(temp, 'www/temp.csv', row.names = FALSE)
   }
 
@@ -723,11 +741,11 @@ server = function(input, output, session) {
   ## Buttons for operations with selection
   startPlay = function() {
     if (!is.null(myPars$myAudio)) {
-      if (!is.null(myPars$spectrogram_brush) &&
-          (myPars$spectrogram_brush$xmax - myPars$spectrogram_brush$xmin > 100)) {
+      if (!is.null(input$spectrogram_brush) &&
+          (input$spectrogram_brush$xmax - input$spectrogram_brush$xmin > 100)) {
         # at least 100 ms selected
-        myPars$play$from = myPars$spectrogram_brush$xmin / 1000
-        myPars$play$to = myPars$spectrogram_brush$xmax / 1000
+        myPars$play$from = input$spectrogram_brush$xmin / 1000
+        myPars$play$to = input$spectrogram_brush$xmax / 1000
       } else {
         myPars$play$from = myPars$spec_xlim[1] / 1000 # myPars$cursor / 1000
         myPars$play$to = myPars$spec_xlim[2] / 1000
@@ -825,11 +843,12 @@ server = function(input, output, session) {
     } else if (button_key == 'PageUp') {     # PageUp (previous file)
       lastFile()
     } else if ((myPars$listen_enter | myPars$listen_enter_edit) & button_key == 'Enter') {
-      if (myPars$listen_enter) {
-        new_annotation()
-      } else {
-        edit_annotation()
-      }
+      # this creates a problem if ENTER is pressed too fast --> label not saved
+      # if (myPars$listen_enter) {
+      #   new_annotation()
+      # } else {
+      #   edit_annotation()
+      # }
     }
   })
 
@@ -978,6 +997,7 @@ server = function(input, output, session) {
     if (!is.null(myPars$out)) {
       # re-order and save a backup
       myPars$out = myPars$out[order(myPars$out$file, myPars$out$from), ]
+      my_annot <<- myPars$out
       write.csv(myPars$out, 'www/temp.csv', row.names = FALSE)
     }
   }
@@ -1020,9 +1040,25 @@ server = function(input, output, session) {
     content = function(filename) {
       done()  # finalize the last file
       write.csv(myPars$out, filename, row.names = FALSE)
+      my_annot <<- myPars$out
       if (file.exists('www/temp.csv')) file.remove('www/temp.csv')
+      # offer to close the app
+      showModal(modalDialog(
+        title = "Terminate the app?",
+        easyClose = TRUE,
+        footer = tagList(
+          actionButton("terminate_no", "Keep working"),
+          actionButton("terminate_yes", "Terminate")
+        )
+      ))
     }
   )
+  observeEvent(input$terminate_no, {
+    removeModal()
+  })
+  observeEvent(input$terminate_yes, {
+    stopApp(returnValue = myPars$out)
+  })
 
   observeEvent(input$about, {
     if (myPars$debugQn) {
@@ -1043,42 +1079,46 @@ server = function(input, output, session) {
   ### TOOLTIPS - have to be here instead of UI b/c otherwise problems with regulating delay
   # (see https://stackoverflow.com/questions/47477237/delaying-and-expiring-a-shinybsbstooltip)
 
-  shinyBS::addTooltip(session, id='reset_to_def', title = 'Reset all settings to default values', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='audioMethod', title = "Play audio with javascript (recommended in Firefox, doesn't work in Chrome) or with R (browser-independent, but then the cursor doesn't move, and you can't stop playback)", placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='reset_to_def', title = 'Reset all settings to default values', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='audioMethod', title = "Play audio with javascript (recommended in Firefox, doesn't work in Chrome) or with R (browser-independent, but then the cursor doesn't move, and you can't stop playback)", placement="right", trigger="hover", options = list(delay = tooltip_options))
 
   # spectrogram
-  shinyBS::addTooltip(session, id='spec_ylim', title = "Range of displayed frequencies, kHz", placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='windowLength', title = 'Length of STFT window, ms.', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='step', title = 'Step between analysis frames, ms', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  # shinyBS::addTooltip(session, id='overlap', title = 'Overlap between analysis frames, %', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='dynamicRange', title = 'Dynamic range, dB', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='spec_cex', title = "Magnification coefficient controlling the size of points showing pitch candidates", placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='spec_yScale', title = 'Frequency scale', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='specContrast', title = 'Regulates the contrast of the spectrogram', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='specBrightness', title = 'Regulates the brightness of the spectrogram', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='zp', title = 'Zero padding: 8 means 2^8 = 256, etc.', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='wn', title = 'Type of STFT window', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='spec_maxPoints', title = 'The number of points to plot in the spectrogram (smaller = faster, but low resolution)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='spec_ylim', title = "Range of displayed frequencies, kHz", placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='windowLength', title = 'Length of STFT window, ms.', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='step', title = 'Step between analysis frames, ms', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='specType', title = 'Spectrogram type, argument "specType" in spectrogram()', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  # shinyBS::addTooltip(session, id='overlap', title = 'Overlap between analysis frames, %', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='dynamicRange', title = 'Dynamic range, dB', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='spec_cex', title = "Magnification coefficient controlling the size of points showing pitch candidates", placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='spec_yScale', title = 'Frequency scale', placement="below", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='specContrast', title = 'Regulates the contrast of the spectrogram', placement="below", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='specBrightness', title = 'Regulates the brightness of the spectrogram', placement="below", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='nColors', title = 'The number of distinct colors in the spectrogram', placement="below", trigger="hover", options = tooltip_options)
+  shinyBS::addTooltip(session, id='blur_freq', title = 'Gaussian filter of frequency: >0 = blur, <0 = unblur (sharpen)', placement="below", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='blur_time', title = 'Gaussian filter of time: >0 = blur, <0 = unblur (sharpen)', placement="below", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='zp', title = 'Zero padding: 8 means 2^8 = 256, etc.', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='wn', title = 'Type of STFT window', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='spec_maxPoints', title = 'The number of points to plot in the spectrogram (smaller = faster, but low resolution)', placement="below", trigger="hover", options = list(delay = tooltip_options))
 
   # oscillogram
-  shinyBS::addTooltip(session, id='osc', title = 'The type of oscillogram to show', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='osc_maxPoints', title = 'The number of points to plot in the oscillogram (smaller = faster, but low resolution)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='osc', title = 'The type of oscillogram to show', placement="below", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='osc_maxPoints', title = 'The number of points to plot in the oscillogram (smaller = faster, but low resolution)', placement="below", trigger="hover", options = list(delay = tooltip_options))
 
   # action buttons
-  shinyBS:::addTooltip(session, id='lastFile', title='Save and return to the previous file (PageUp)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS:::addTooltip(session, id='nextFile', title='Save and proceed to the next file (PageDown)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS:::addTooltip(session, id='selection_stop', title='Stop playback', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS:::addTooltip(session, id='selection_play', title='Play selection (SPACEBAR)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS:::addTooltip(session, id='selection_annotate', title='Create a new annotation (A or DOUBLE-CLICK)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS:::addTooltip(session, id='selection_delete', title='Remove annotation (DELETE / BACKSPACE)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='saveRes', title = 'Download results (see ?pitch_app for recovering unsaved data after a crash)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS:::addTooltip(session, id='lastFile', title='Save and return to the previous file (PageUp)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS:::addTooltip(session, id='nextFile', title='Save and proceed to the next file (PageDown)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS:::addTooltip(session, id='selection_stop', title='Stop playback', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS:::addTooltip(session, id='selection_play', title='Play selection (SPACEBAR)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS:::addTooltip(session, id='selection_annotate', title='Create a new annotation (A or DOUBLE-CLICK)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS:::addTooltip(session, id='selection_delete', title='Remove annotation (DELETE / BACKSPACE)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='saveRes', title = 'Download results (see ?pitch_app for recovering unsaved data after a crash)', placement="right", trigger="hover", options = list(delay = tooltip_options))
 
   # navigation / zoom
-  shinyBS::addTooltip(session, id='zoomIn_freq', title = 'Zoom in frequency (+)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='zoomOut_freq', title = 'Zoom out frequency (-)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='scrollLeft', title = 'Scroll left (arrow LEFT)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='zoomOut', title = 'Zoom out time (arrow DOWN)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='zoomToSel', title = 'Zoom to selection (S)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='zoomIn', title = 'Zoom in time (arrow UP)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='scrollRight', title = 'Scroll right (arrow RIGHT)', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
+  shinyBS::addTooltip(session, id='zoomIn_freq', title = 'Zoom in frequency (+)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='zoomOut_freq', title = 'Zoom out frequency (-)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='scrollLeft', title = 'Scroll left (arrow LEFT)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='zoomOut', title = 'Zoom out time (arrow DOWN)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='zoomToSel', title = 'Zoom to selection (S)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='zoomIn', title = 'Zoom in time (arrow UP)', placement="right", trigger="hover", options = list(delay = tooltip_options))
+  shinyBS::addTooltip(session, id='scrollRight', title = 'Scroll right (arrow RIGHT)', placement="right", trigger="hover", options = list(delay = tooltip_options))
 }
